package glzt.ui.video;

import glzt.common.unit.DateTimeUntil;
import glzt.common.unit.ServiceMsg;
//import gl.inet.common.ServiceMsg;

import java.io.EOFException;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class MonitorView extends SurfaceView implements SurfaceHolder.Callback {

	public final static int SIZE_STANDARD = 1;
	public final static int SIZE_BEST_FIT = 4;
	public final static int SIZE_FULLSCREEN = 8;

	private MjpegViewThread thread;
	private VideoMJPGStream mIn;
	private boolean mRun = false;
	private boolean surfaceDone = false;
	private int dispWidth;
	private int dispHeight;
	private int displayMode;
	private String url;
	private Handler handler;
	private static boolean isSessionStart = true;
	private boolean isChangeSetting = false;
	private boolean blnDisplay = false;

	public MonitorView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}

	public void surfaceCreated(SurfaceHolder holder) {
		dispWidth = getWidth();
		dispHeight = getHeight();
		surfaceDone = true;

		this.startPlayback();
	}

	public void surfaceChanged(SurfaceHolder holder, int f, int w, int h) {
		if (thread != null)
			thread.setSurfaceSize(w, h);

	}

	public void surfaceDestroyed(SurfaceHolder holder) {
		surfaceDone = false;
		this.stopPlayback();
		return;
	}

	public MonitorView(Context context) {
		super(context);
		init(context);
	}

	public class MjpegViewThread extends Thread {
		private SurfaceHolder mSurfaceHolder;
		private boolean isDay;

		public MjpegViewThread(SurfaceHolder surfaceHolder, Context context,
				boolean isDay) {
			mSurfaceHolder = surfaceHolder;
			this.isDay = isDay;

		}

		private Rect destRect(int bmw, int bmh) {
			int tempx;
			int tempy;
			if (displayMode == MonitorView.SIZE_STANDARD) {
				tempx = (dispWidth / 2) - (bmw / 2);
				tempy = (dispHeight / 2) - (bmh / 2);
				return new Rect(tempx, tempy, bmw + tempx, bmh + tempy);
			}
			if (displayMode == MonitorView.SIZE_BEST_FIT) {
				float bmasp = (float) bmw / (float) bmh;
				bmw = dispWidth;
				bmh = (int) (dispWidth / bmasp);
				if (bmh > dispHeight) {
					bmh = dispHeight;
					bmw = (int) (dispHeight * bmasp);
				}
				tempx = (dispWidth / 2) - (bmw / 2);
				tempy = (dispHeight / 2) - (bmh / 2);
				return new Rect(tempx, tempy, bmw + tempx, bmh + tempy);
			}
			if (displayMode == MonitorView.SIZE_FULLSCREEN)
				return new Rect(0, 0, dispWidth, dispHeight);
			return null;
		}

		public void setSurfaceSize(int width, int height) {
			synchronized (mSurfaceHolder) {
				dispWidth = width;
				dispHeight = height;
			}
		}

		@Override
		public void run() {

			{

				mIn = VideoMJPGStream.read(url);
				Log.i("runMonitor", "read url==" + url);
				Log.i("runMonitor", "read mIn==" + mIn);
				if (mIn == null) {
					sendMessageToUI(ServiceMsg.ON_MSG_CONNECT_VIDEO_INFO,
							false, 0, "");
					mRun = false;
					return;
				}

				Bitmap bm = null;
				Rect destRect;
				Canvas c = null;
				Paint p = new Paint();

				while (mRun) {
					Log.i("checked", "blnDisplay ==" + blnDisplay);
					if (blnDisplay) {
						startPlayback();
					}

					if (!blnDisplay) {
						stopPlayback();
					}

					if (!surfaceDone) {
						continue;
					}

					try {
						bm = mIn.readMjpegFrame();
						Log.i("runMonitor", "read bm==" + bm);
						if (isSessionStart) {
							sendMessageToUI(
									ServiceMsg.ON_MSG_CONNECT_VIDEO_INFO, true,
									0, "");
							isSessionStart = false;
						}
					} catch (Exception e) {
						String excpDes = "";
						if (e != null)
							excpDes = e.getLocalizedMessage();

						if (excpDes != null
								&& !excpDes.equalsIgnoreCase("Socket closed")
								&& !isChangeSetting()) {
							sendMessageToUI(
									ServiceMsg.ON_MSG_CONNECT_VIDEO_INFO, true,
									0, "");
							VideoService.getInstance().restart(url);
						} else {
							isSessionStart = true;
							mIn = VideoMJPGStream.read(url);
							continue;
						}
					}

					if (bm == null)
						continue;

					c = mSurfaceHolder.lockCanvas();
					synchronized (mSurfaceHolder) {
						try {
							destRect = destRect(bm.getWidth(), bm.getHeight());
							c.drawColor(Color.LTGRAY);
							c.drawBitmap(bm, null, destRect, p);
							bm.recycle();
							bm = null;
						} catch (Exception e) {
						} finally {
							if (c != null)
								mSurfaceHolder.unlockCanvasAndPost(c);

						}
					}
				}
			}
		}
	}

	public void setSessionStart(boolean hold) {
		this.isSessionStart = hold;
	}

	public void init(Context context) {
		SurfaceHolder holder = getHolder();
		holder.addCallback(this);

		// setFocusable(true);
		setFocusable(false);
		setFocusableInTouchMode(false);
		displayMode = MonitorView.SIZE_STANDARD;
	}

	private void sendMessageToUI(int msgId, boolean result, int indeication,
			String description) {
		assert (handler != null);
		Message msg = handler.obtainMessage(msgId);
		Bundle bundle = new Bundle();
		bundle.putInt("indication", indeication);
		bundle.putBoolean("success", result);
		bundle.putString("description", description); //
		msg.setData(bundle);
		handler.sendMessage(msg);
	}

	public void setHandler(Handler handler) {
		this.handler = handler;
	}

	public void setSource(String url) {
		this.url = url;
	}

	public void stopPlayback() {
		new Thread(new Runnable() {
			@Override
			public void run() {
				mRun = false;
				boolean retry = true;
				while (retry) {
					try {
						if (mIn != null)
							mIn.close();
						retry = false;
					} catch (Exception e) {
						if (e.getLocalizedMessage() != null)
							Log.d("MjpgVieoView.stopPlayback:",
									e.getLocalizedMessage());
					} finally {
						mIn = null;
					}
				}
			}
		}).start();

	}

	public void startPlayback() {
		if (mRun || url == null) {
			return;
		}

		boolean isDay = DateTimeUntil.isDay();

		mRun = true;
		SurfaceHolder holder = getHolder();
		thread = new MjpegViewThread(holder, getContext(), isDay);
		thread.start();
	}

	public void closeStream() {
		if (mIn != null)
			try {
				mIn.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}

	public void setDisplayMode(int s) {
		displayMode = s;
	}

	public boolean isChangeSetting() {
		return isChangeSetting;
	}

	public void setChangeSetting(boolean isChangeSetting) {
		this.isChangeSetting = isChangeSetting;
	}

	public void setBlnDisplayCamra(boolean blnDisplay) {
		this.blnDisplay = blnDisplay;
	}

}